<?php

/**
 * Return JSON in OpenAPI 3.0 format.
 */
function rest_api_doc_openapi() {
  $resources = restws_get_resource_info();
  $components = [];
  $components['securitySchemes']['BasicAuth'] = ['type' => 'http', 'scheme' => 'basic'];
  $paths = [];

  foreach ($resources as $resource_path => $resource) {

    if (!in_array($resource_path, ['node', 'user', 'profile2'])) {
      continue;
    }

    $tags[] = [
      'name' => $resource['label'],
      'description' => t('Manage @label entities.', ['@label' => $resource['label']]),
    ];

    $controller = restws_resource_controller($resource_path);

    foreach ($controller->propertyInfo() as $property => $property_info) {
      if (!empty($property_info['computed'])) {
        continue;
      }

      $openApiType = $openApiFormat = '';

      if (empty($property_info['type']) && !empty($property_info['schema field'])) {
        $openApiType = 'string';
      }
      elseif (in_array($property_info['type'], ['integer'])) {
        $openApiType = 'integer';
      }
      elseif (in_array($property_info['type'], ['positive_integer'])) {
        $openApiType = 'integer';
      }
      elseif (in_array($property_info['type'], ['decimal'])) {
        $openApiType = 'integer';
        $openApiFormat = 'float';
      }
      elseif (in_array($property_info['type'], ['boolean'])) {
        $openApiType = 'boolean';
      }
      elseif (in_array($property_info['type'], ['string', 'text_formatted', 'text'])) {
        $openApiType = 'string';
      }
      elseif ($property_info['type'] == 'token') {
        $openApiType = 'string';
      }
      elseif (entity_get_info($property_info['type'])) {
        // This is a reference to an entity.
        $openApiType = 'integer';
      }
      elseif ($property_info['type'] == 'entity') {
        // This is a reference to an entity.
        $openApiType = 'integer';
      }
      elseif ($property_info['type'] == 'date') {
        // Ugh, handle this mess.
        $openApiType = 'string';
        $openApiFormat = 'date-time';
      }
      elseif ($property_info['type'] == 'duration') {
        // Ugh, handle this mess.
        $openApiType = 'integer';
      }
      elseif ($property_info['type'] == 'struct') {
        //$openApiType = 'object';
        continue;
      }
      elseif (strpos($property_info['type'], '<') !== FALSE) {
        // Example: list<text>
        //$openApiType = 'array';
        continue;
      }
      else {
        continue;
      }

      // We do not care about feeds.
      if (strpos($property, 'feeds_') === 0) {
        continue;
      }
      if (strpos($property, 'feed_') === 0) {
        continue;
      }
      if ($property == 'vid') {
        continue;
      }

      $components['schemas'][$resource_path]['properties'][$property]['type'] = $openApiType;
    }

    $paths['/' . $resource_path . '.json'] = [
      'get' => [
        'tags' => [$resource['label']],
        'summary' => t('Returns a list of @label.', ['@label' => $resource['label']]),
        'responses' => [
          '200' => [
            'description' => t('Returns a list of @label.', ['@label' => $resource['label']]),
          ],
        ],
      ],
    ];
    $paths['/' . $resource_path . "/{{$resource_path}}.json"] = [
      'get' => [
        'tags' => [$resource['label']],
        'summary' => t('Get a @label.', ['@label' => $resource['label']]),
        'parameters' => [
          [
            'in' => 'path',
            'name' => $resource_path,
            'schema' => ['type' => 'integer'],
            'required' => 'true',
            'description' => t('Numeric ID of the @label to get.', ['@label' => $resource['label']]),
          ],
        ],
        'responses' => [
          '200' => [
            'content' => [
              'application/json' => [
                'schema' => [
                  '$ref' => '#/components/schemas/' . $resource_path,
                ]
              ],
            ],
            'description' => t('Get a @label.', ['@label' => $resource['label']]),
          ],
        ],
      ],
    ];
    $paths['/' . $resource_path] = [
      'post' => [
        'tags' => [$resource['label']],
        'summary' => t('Create a new @label.', ['@label' => $resource['label']]),
        'requestBody' => [
          'required' => true,
          'content' => [
            'application/json' => [
              'schema' => [
                '$ref' => '#/components/schemas/' . $resource_path,
              ]
            ],
          ],
        ],
        'responses' => [
          '200' => [
            'description' => t('The created @label.', ['@label' => $resource['label']]),
          ],
        ],
      ],
    ];

    $paths['/' . $resource_path . "/{{$resource_path}}"] = [
      'put' => [
        'tags' => [$resource['label']],
        'summary' => t('Update a @label.', ['@label' => $resource['label']]),
        'parameters' => [
          [
            'in' => 'path',
            'name' => $resource_path,
            'schema' => ['type' => 'integer'],
            'required' => 'true',
            'description' => t('Numeric ID of the @label to get.', ['@label' => $resource['label']]),
          ],
        ],
        'responses' => [
          '200' => [
            'description' => t('The updated @label.', ['@label' => $resource['label']]),
          ],
        ],
      ],
      'delete' => [
        'tags' => [$resource['label']],
        'summary' => t('Delete a @label.', ['@label' => $resource['label']]),
        'parameters' => [
          [
            'in' => 'path',
            'name' => $resource_path,
            'schema' => ['type' => 'integer'],
            'required' => 'true',
            'description' => t('Numeric ID of the @label to delete.', ['@label' => $resource['label']]),
          ],
        ],
        'responses' => [
          '200' => [
            'description' => t('Empty response.', ['@label' => $resource['label']]),
          ],
        ],
      ],
    ];

    $json = [
      'openapi' => '3.0.0',
      'info' => [
        'title' => variable_get('site_name') ,
        'description' => t('OpenAPI 3.0 specification.'),
        'version' => '1.0.0',
        'host' => $_SERVER['HTTP_HOST'],
        'basePath' => '',
      ],
      'servers' => [
        [
          'url' => rtrim(url('/', ['absolute' => TRUE]), '/'),
        ],
      ],
      'paths' => $paths,
      'components' => $components,
      'tags' => $tags,
    ];
  }

  header('Content-type: application/json');
  print json_encode($json, JSON_PRETTY_PRINT);
  exit;
}
